# Generated by Django 5.1 on 2024-06-13 10:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0028_squashed"),
        ("lnschema_core", "0054_alter_feature_previous_runs_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cellline",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="disease",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="gene",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="organism",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="protein",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="publicsource",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="previous_runs",
            field=models.ManyToManyField(related_name="+", to="lnschema_core.run"),
        ),
    ]
